import csv
import time
from util import *
from datetime import datetime,timedelta
import numpy as np
import matplotlib.dates as mdates
import matplotlib.pyplot as plt
from matplotlib.lines import Line2D
from scipy import stats
from scipy.stats import linregress
from scipy.ndimage import filters
import random

print("Loading Data")
tic = time.perf_counter()
data = load_data(['RADAR', 'FTP'])
toc = time.perf_counter()
print(f"Loaded in: {toc - tic:0.4f} s")

class ZBiasFreePlotter(object):
    def __init__(self, active = True):
        self.plot_calls = []
        self.active = active

    def add_plot(self, f, xs, ys, *args, **kwargs):
        self.plot_calls.append((f, xs, ys, args, kwargs))

    def draw_plots(self, chunk_size=512):
        if self.active:
            scheduled_calls = []
            for f, xs, ys, args, kwargs in self.plot_calls:
                xs = np.array(xs)
                ys = np.array(ys)
                assert(len(xs) == len(ys))
                index = np.arange(len(xs))
                np.random.shuffle(index)
                index_blocks = [index[i:i+chunk_size] for i in np.arange(len(index))[::chunk_size]]
                for i, index_block in enumerate(index_blocks):
                    if i != 0 and kwargs.get("label") is not None:
                        kwargs = kwargs.copy()
                        kwargs["label"] = None
                    scheduled_calls.append((f, xs[index_block], ys[index_block], args, kwargs))

            np.random.shuffle(scheduled_calls)

            for f, xs, ys, args, kwargs in scheduled_calls:
                f(xs, ys, *args, **kwargs)
        else:
            for f, xs, ys, args, kwargs in self.plot_calls:
                f(xs, ys, *args, **kwargs)

def line(l, x):
    return l.slope*x + l.intercept

def filter(data):
    date = (datetime(2023,1,12,0) < data.time < datetime(2023,2,28,0))
    
    hach_depth = data.hach_depth > 0.10
    
    df = data.radar_fine_depth[0][0]
    ang = math.radians(data.radar_angle)
    radar_depth = (0 < 1340 - df*math.sin(ang)) and df > 350 #1340mm radar installation height
    
    angle = (70 < data.radar_angle < 87)
    
    return date and angle and hach_depth and radar_depth


data = [x for x in data if filter(x)]

t = [x.time for x in data]
b = [x.radar_battery for x in data]
p = [x.radar_angle for x in data]
v = [x.radar_ftp_vel for x in data]
vr = [x.radar_vel for x in data]
a = [x.radar_amp for x in data]
s = [x.radar_spread for x in data]
d = [x.radar_ftp_depth for x in data]
dr = [x.radar_depth for x in data]
df = [x.radar_fine_depth[0][0] for x in data]
dff = [x.radar_fine_depth[0][0] for x in data]
vh = [x.hach_vel for x in data]
dh = [1000*x.hach_depth for x in data]
swp = [x.radar_max_bin for x in data]

v = [vi/math.cos(math.radians(pi)) for vi,pi in zip(v,p)]
vr = [vi/math.cos(math.radians(pi)) for vi,pi in zip(vr,p)]
vt = [vfi if ti < datetime(2022,10,4) else vri for vfi,vri,ti in zip(v,vr,t)]
d = [1340 - di*math.sin(math.radians(pi)) for di,pi in zip(d,p)]
df = [1340 - di*math.sin(math.radians(pi)) for di,pi in zip(df,p)]
dr = [1340 - di*math.sin(math.radians(pi)) for di,pi in zip(dr,p)]


fig = plt.figure(figsize=(7.16, 8), dpi = 94) 
rng_z_plt = ZBiasFreePlotter(False)
plt.rcParams.update({'font.size': 12})

axrow = 2
axcol = 2
gs = fig.add_gridspec(axrow, axcol, hspace = 0.1, wspace = 0.02, width_ratios = [4,9],
                      bottom = 0.12, top = 0.97, left = 0.12, right = 0.97)
ax = fig.add_subplot(gs[1,1]) 
ax_l = fig.add_subplot(gs[1,0]) 
ax2 = fig.add_subplot(gs[0,1]) 
ax2_l = fig.add_subplot(gs[0,0]) 

ax.grid()
ax_l.grid()
ax2.grid()
ax2_l.grid()

ax.set_axisbelow(True)
ax_l.set_axisbelow(True)
ax2.set_axisbelow(True)
ax2_l.set_axisbelow(True)

point_size = 3
trans = 1


rng_z_plt.add_plot(ax.scatter,t,vh, c = 'dodgerblue', s = point_size,alpha=trans, label = 'hach')
rng_z_plt.add_plot(ax_l.scatter,t,vh, c = 'dodgerblue', s = point_size,alpha=trans, label = 'hach')
rng_z_plt.add_plot(ax.scatter,t,vr, c = 'darkorchid', s = point_size,alpha=trans, label = 'radar')
rng_z_plt.add_plot(ax_l.scatter,t,vr, c = 'darkorchid', s = point_size,alpha=trans, label = 'radar')


rng_z_plt.add_plot(ax2.scatter, t,dh, c = 'dodgerblue', s = point_size,alpha=trans, label = 'hach')
rng_z_plt.add_plot(ax2_l.scatter, t,dh, c = 'dodgerblue', s = point_size,alpha=trans, label = 'hach')
rng_z_plt.add_plot(ax2.scatter, t,df, c = 'darkorchid', s = point_size,alpha=trans, label = 'radar')
rng_z_plt.add_plot(ax2_l.scatter, t,df, c = 'darkorchid', s = point_size,alpha=trans, label = 'radar')

rng_z_plt.draw_plots()

ax_l.set_ylabel('Velocity (mm/s)');
ax2_l.set_ylabel('Depth (mm)');
ax2.set_xlabel('Date', x=0.25)


legend_elements = [Line2D([0], [0], marker='o', color='dodgerblue', label='hach', markersize=5, lw=0),
                   Line2D([0], [0], marker='o', color='darkorchid', label='radar', markersize=5, lw=0)]
ax.legend(handles=legend_elements)
ax2.legend(handles=legend_elements)

vlim = 1200
dlim = 600

ax.set_ylim([0,vlim]);
ax_l.set_ylim([0,vlim]);
ax2.set_ylim([0,dlim]);
ax2_l.set_ylim([0,dlim]);

dstart_l = datetime(2023,1,17,12)
dend_l = datetime(2023,1,19,12)
dstart = datetime(2023,2,2,12)
dend = datetime(2023,2,7,0)

ax.spines['left'].set_visible(False)
ax_l.spines['right'].set_visible(False)
ax_l.yaxis.tick_left()
ax_l.tick_params(axis='y', which='both', left=True, right=False, labelleft=True)
ax.tick_params(axis='y', which='both', left=False, right=False, labelleft=False)

ax2.spines['left'].set_visible(False)
ax2_l.spines['right'].set_visible(False)
ax2_l.yaxis.tick_left()
ax2_l.tick_params(axis='y', which='both', left=True, right=False, labelleft=True)
ax2.tick_params(axis='y', which='both', left=False, right=False, labelleft=False)


ax.set_xlim(dstart, dend) 
ax_l.set_xlim(dstart_l, dend_l) 
ax2.set_xlim(dstart, dend) 
ax2_l.set_xlim(dstart_l, dend_l) 

ax.xaxis.set_major_locator(mdates.DayLocator(interval=1))
ax_l.xaxis.set_major_locator(mdates.DayLocator(interval=1))
ax2.xaxis.set_major_locator(mdates.DayLocator(interval=1))
ax2_l.xaxis.set_major_locator(mdates.DayLocator(interval=1))

fig.autofmt_xdate()

s = 4/9
d = .015 # how big to make the diagonal lines in axes coordinates

kwargs = dict(transform=ax_l.transAxes, color='k', clip_on=False)
ax_l.plot((1-d,1+d), (-d,+d), **kwargs)
ax_l.plot((1-d,1+d),(1-d,1+d), **kwargs)

kwargs.update(transform=ax.transAxes)  # switch to the bottom axes
ax.plot((-d*s,+d*s), (1-d,1+d), **kwargs)
ax.plot((-d*s,+d*s), (-d,+d), **kwargs)

kwargs = dict(transform=ax2_l.transAxes, color='k', clip_on=False)
ax2_l.plot((1-d,1+d), (-d,+d), **kwargs)
ax2_l.plot((1-d,1+d),(1-d,1+d), **kwargs)

kwargs.update(transform=ax2.transAxes)  # switch to the bottom axes
ax2.plot((-d*s,+d*s), (1-d,1+d), **kwargs)
ax2.plot((-d*s,+d*s), (-d,+d), **kwargs)

plt.savefig('timeseries.png', dpi = 300, transparent = True)
plt.show()


###########correlation #########################################

lobf = linregress(dh,df)
print("depth: ", lobf)

lobf_v = linregress(vh,vr)
print("velocity: ", lobf_v)

fig = plt.figure(figsize=(7.16, 3.5), dpi = 94) 
plt.rcParams.update({'font.size': 12})

axrow = 1
axcol = 2

gs = fig.add_gridspec(axrow, axcol, wspace = 0.35, bottom = 0.12, top = 0.97, left = 0.12, right = 0.97)
ax = fig.add_subplot(gs[0,0]) 
ax2 = fig.add_subplot(gs[0,1]) 

ax.grid()
ax2.grid()

ax.set_axisbelow(True)
ax2.set_axisbelow(True)

vlim = 1200
dlim = 600

ax.plot([-100,dlim], [-100,dlim], linestyle='--', color='grey', zorder = 10)
ax.plot([-100,dlim], line(lobf,np.array([-100,dlim])), color = 'k', zorder = 12)
ax.scatter(dh,df,  c = 'royalblue', s = 1,alpha=0.4, zorder = 14)

ax2.plot([-100,vlim], [-100,vlim], linestyle='--', color='grey', zorder = 10)
ax2.plot([-100,vlim], line(lobf_v,np.array([-100,vlim])), color = 'k', zorder = 12)
ax2.scatter(vh,vr,  c = 'royalblue', s = 3,alpha=0.8, zorder = 14)


ax.set_ylim([0,dlim])
ax.set_xlim([0,dlim])

ax2.set_ylim([0,vlim])
ax2.set_xlim([0,vlim])

ax.set_aspect('equal')
ax2.set_aspect('equal')

dticks = [0, 100, 200, 300, 400, 500, 600]
ax.set_xticks(dticks)
ax.set_yticks(dticks)

vticks = [0, 200, 400, 600, 800, 1000, 1200]
ax2.set_xticks(vticks)
ax2.set_yticks(vticks)


ax.set_ylabel('Radar Depth (mm)');
ax.set_xlabel('HACH Depth (mm)')
ax2.set_ylabel('Radar Velocity (mm/s)');
ax2.set_xlabel('HACH Velocity (mm/s)')

plt.savefig('correlation.png', dpi = 300, transparent = True)
plt.show()











