import csv
import pickle
from dataclasses import dataclass
from os import listdir
from os.path import isfile, join, getmtime
import math
from datetime import datetime,timedelta
import numpy as np
from scipy.ndimage import gaussian_filter as gf

import matplotlib.pyplot as plt


@dataclass
class datapoint:       
        time = datetime(1, 1, 1)
        radar_battery = 0
        radar_vel = 0
        radar_ftp_vel = 0
        radar_depth = 0
        radar_ftp_depth = 0
        radar_amp = 0
        radar_spread = 0
        radar_angle = 0
        radar_max_bin = []
        radar_fine_depth = []
        hach_vel = 0
        hach_depth = 0


def load_hach(hach_f):
    ##lists to store data from hach files
    hach_date = []
    hach_depth = []
    hach_vel = []
    with open(hach_f, newline='') as csvfile:
        csvreader = csv.reader(csvfile, delimiter=',', quotechar='|')
        for i,row in enumerate(csvreader):
            if i < 5:
                    continue
            date_string = (row[0] + " " + row[1]).replace('"','')
            hach_date.append(datetime.strptime(date_string, '%d/%b/%y %H:%M'))
            hach_depth.append(float(row[2]))
            hach_vel.append(float(row[3].strip('*'))*1000)
    return [hach_date, hach_depth, hach_vel]
##enum for the above list
DATE = 0
DEPTH = 1
VEL = 2
SWEEP = 3
########################

def decode_fine(stng):


    splits = stng.split('|')

    f_dists = []
    for split in splits:
        try:
            dist = int(split[:-4], 0x10)
            amp = int(split[-4:], 0x10)
        except ValueError:
            dist = 0
            amp = 0

        f_dists.append((dist,amp))
    while len(f_dists) < 4:
        f_dists.append((0,0))
    
    return f_dists

def load_ftp(ftp_f):
    ##lists to store data from hach files
    ftp_date = []
    ftp_depth = []
    ftp_vel = []
    ftp_sweep = []
    with open(ftp_f, newline='') as csvfile:
        csvreader = csv.reader(csvfile, delimiter=',', quotechar='|')
        for i,row in enumerate(csvreader):
            date_string = row[0]
            ftp_date.append(datetime.utcfromtimestamp(int(date_string, 16))+timedelta(hours=10))
            ftp_depth.append(float(row[35]))
            ftp_vel.append(float(row[34]))
            ftp_sweep.append([float(row[i]) for i in range(1,33)])
    return [ftp_date, ftp_depth, ftp_vel, ftp_sweep]

def load_radar(radar_fs):
    points = []
    for radar_f in radar_fs:
        with open(radar_f, newline='') as csvfile:
            csvreader = csv.reader(csvfile, delimiter=',', quotechar='|')
            for i,row in enumerate(csvreader):
                try:
                    point = datapoint()
                    try:
                        point.time = datetime.strptime(row[0], '%d/%m/%y %I:%M:%S %p')+timedelta(hours=10)
                    except ValueError:
                        point.time = datetime.strptime(row[0], '%d/%m/%Y %H:%M')+timedelta(hours=10)
                    try:
                        point.radar_battery = float(row[4])  
                    except ValueError:
                        pass
                    try:
                        point.radar_angle = float(row[5]) 
                    except ValueError:
                        pass
                    try:
                        point.radar_vel = float(row[6]) 
                    except ValueError:
                        pass
                    try:    
                        point.radar_fine_depth = decode_fine(row[7])
                    except ValueError:
                        pass
                    try:    
                        point.radar_depth = float(row[8]) 
                    except ValueError:
                        pass
                    try:    
                        point.radar_amp = float(row[9])
                    except ValueError:
                        pass
                    try:
                        point.radar_spread = float(row[10])
                    except ValueError:
                        pass
    
                    points.append(point)
                except ValueError:
                    pass  
    points.sort(key = lambda x : x.time)
    return points

def match_data(radardata, auxdata, hach):
    hi = 0;
    skip = False
   
    for point in radardata:          
        try:
            boslT = point.time
            auxT  = auxdata[DATE][hi]
            Dtime = boslT - auxT

            while (Dtime >= timedelta(minutes=1)):
                hi += 1
                boslT = point.time
                auxT  = auxdata[DATE][hi]
                Dtime = boslT - auxT

            while (Dtime <= timedelta(minutes=-1)):
                hi -= 1
                boslT = point.time
                auxT  = auxdata[DATE][hi]
                Dtime = boslT - auxT
                
                if hi < 0:
                    hi = 0
                    skip = True
                    break;
            
            if hach:
                point.hach_vel = auxdata[VEL][hi]
                point.hach_depth = auxdata[DEPTH][hi]
            else:
                point.radar_ftp_vel = auxdata[VEL][hi]
                point.radar_ftp_depth = auxdata[DEPTH][hi]
                point.radar_max_bin = auxdata[SWEEP][hi]
                
        except IndexError:
            break

        if skip:
            skip = False
            continue
            
    return radardata

def match_data_sources(radardata, hachdata, ftpdata):
    

    radardata = match_data(radardata, hachdata, True)
    if ftpdata != None:
        radardata = match_data(radardata, ftpdata, False)
    
    return radardata

def data_cached(fp_cache, source_files):   
    if (not isfile(fp_cache)):
        print("No Cache File")
        return False
    else:
        cache_time = getmtime(fp_cache)
        source_times = [getmtime(f) for f in source_files if f != None]
        if (cache_time > max(source_times)):
            return True
        else:
            print("Cache File Out of Date")
            return False
        
def write_cache(fp_cache, datapoints):
    print('Writing Cache')
    with open(fp_cache, 'wb') as cache:
        pickle.dump(datapoints, cache, pickle.HIGHEST_PROTOCOL)

def read_cache(fp_cache):
    print('Reading Cache')
    with open(fp_cache, 'rb') as cache:
        datapoints = pickle.load(cache)
            
    return datapoints

def get_fp(sensors):
    fps = []
    if 'RADAR' in sensors:
        fps.append('data/RADAR/radar.csv')
    # if 'FTP' in sensors:
        # fps.append('data/RADAR/radar.csv')
    return fps
    
def load_data(sensors):

    hach_f = 'data/HACH_RIGHT/right.csv'
    # hach_f = 'data/HACH_MID/mid.csv'
    ftp_f = 'data/FTP/ftp.csv' if 'FTP' in sensors else None
    radar_fs = get_fp(sensors)
    sd_f = None
    ftpdata = None
    
    fp_cache = 'data/cache_%s.pkl' % str(radar_fs).replace("/", "-")
        
    point_bach = []
    
    if(data_cached(fp_cache, [hach_f,ftp_f] + radar_fs)):
        datapoints = read_cache(fp_cache)
    else:
        if(hach_f != None):       
            hachdata = load_hach(hach_f) 
            
        if(ftp_f != None):       
            ftpdata = load_ftp(ftp_f) 
            
        if(radar_fs != []):
            radardata = load_radar(radar_fs)
        
        datapoints = match_data_sources(radardata,hachdata,ftpdata)
        write_cache(fp_cache,datapoints)
        
    return datapoints



def load_sd():
    if(sd_f != 0):
        hi = 0;
        skip = False
        with open(sd_f, newline='') as csvfile:
            spamreader = csv.reader(csvfile, delimiter=',', quotechar='|')
            for i,row in enumerate(spamreader):
                try:
                    idsd = float(row[0])
                    vemsd = float(row[136])
                    vessd = float(row[137])
                except IndexError:
                    continue
                
                try:
                    point = point_bach[hi]
                
                    
                    while(point.get_id() < idsd):
                        hi += 1;
                        point = point_bach[hi]
                

                    while(point.get_id() > idsd):
                        hi -= 1;
                        point = point_bach[hi]

                        if hi < 0:
                                hi = 0
                                skip = True
                                break;  
                except IndexError:
                    pass

                if skip:
                    skip = False
                    continue
                
                if (point.match(idsd,vemsd,vessd)):
                    for j in range(128):
                            point.set_fft(float(row[j+3]),j)
                    point.set_scale(float(row[132]))